# -*- coding: UTF-8 -*-

#   icd_grp.py      2020.7.27   kita

#!! グリッパー開閉用円盤板カム作図Script

# 2021.5.5  外接するカム曲線にも対応
# 2023.7.12 R回転ホイールでのバグ修正（今頃？）

import tkinter as tk
from tkinter import *
from tkinter import ttk
import universal_cam as uvc
import numpy as np
import my_vector as myv
import zen_han      #全角<-->半角変換
import subprocess
import os
import icd_module as imd

import clr                  # .NETをimport
clr.AddReference('sxnet')   # 
from sxnet import *     # 全てのクラスをimport
SxSys.init(3999)

# Editorの設定
fn = 'editor.ini'

if os.path.exists(fn):      # fnが存在するなら
    fp = open(fn, 'r')
    for line in fp.readlines():
        if line.find('#') > -1:  # #が含まれてるなら
            continue
        line = line.rstrip()    # 行末の改行を削除
        tmp = line.split(',')
        if tmp[0] == 'editor':
            editor = tmp[1]
else:
    editor = 'notepad'

#-------------
# 各種関数
#-------------

def get_cam():  # Cam No 、作図条件取得
    cam_nm = v_cam.get()
    if cam_nm.find('変形正弦') > -1:
        cam_no = 26
    elif  cam_nm.find('変形台形') > -1:
        cam_no = 25
    elif  cam_nm.find('変形等速度') > -1:
        cam_no = 27
    elif  cam_nm.find('Cycroidal') > -1:
        cam_no = 22
    cam = uvc.Universal_cam(cam_no)
    th_h = float(e_th_h.get())
    tau_h = float(e_tau_h.get())
    rpm = float(e_rpm.get())
    layer = int(e_layer.get())  
    #layer = float(e_layer.get()) #整数じゃないとエラー
    offset = float(e_offset.get())

    return (th_h, tau_h, rpm, layer, cam, offset)


def push_help():
    cmd = editor + ' icd_grp.txt'
    result = subprocess.call(cmd,shell=True)



def push_graph():
    (th_h, tau_h, rpm, layer, cam, offset) = get_cam()  # 諸条件とcam Object 取得
    all_sva = cam.sva_nn(100,1)

def print_inf(ss_max_ang, ss_max_accel):
    opt = SxOptCreateText() # 文字列要素作成オプション
    (th_h, tau_h, rpm, layer, cam, offset) = get_cam()  # 諸条件とcam Object 取得
    cam_nm = v_cam.get()
    #max_ang = e_max_ang.get()
    #max_accel = e_max_accel.get()

    # 文字列情報を設定
    ss = []
    ss.append('割付角：    %3.2f [deg]' % th_h)
    ss.append('揺動角：    %3.2f [deg]' % tau_h)
    ss.append('ホイル回転数： %3.2f [rpm]' % rpm)
    ss.append('Cam曲線：  %s' % cam_nm)
    ss.append('最大圧力角：  %s [deg]' % ss_max_ang)
    ss.append('最大加速度： %s [rad/sec2]' % ss_max_accel)
    for i in range(len(ss)):
        tmp = zen_han.Han2Zen(ss[i])
        opt.text[i] = tmp
  
    print('\n作図条件書出点を指示してください')
    pos1 = SxSys.getPos()   # 配置位置
    seg = SxEntSeg.createText(pos1, opt)
    #print('\n配置完了しました')


def push_th2tau():
    (th_h, tau_h, rpm, layer, cam, offset) = get_cam()  # 諸条件とcam Object 取得
    th = float(e_th.get())
    t = th/th_h
    (s, v, a) = cam.sva(t)
    ss = '%3.5f' % (s*tau_h)
    e_tau.delete(0,tk.END)  # エントリー中身を削除
    e_tau.insert(0,ss)     # エントリーの中身を設定

def push_tau2th():
    (th_h, tau_h, rpm, layer, cam, offset) = get_cam()  # 諸条件とcam Object 取得
    tau = float(e_tau.get())
    target_s = tau/tau_h
    target_t = cam.s2t(target_s)        # default gosa0 = 0.0001
    if target_t == None:
        print('不正な数値です')
        return

    ss = '%3.5f' % (target_t*th_h)
    e_th.delete(0,tk.END)  # エントリー中身を削除
    e_th.insert(0,ss)     # エントリーの中身を設定
    

def push_sakuzu():  # カム曲線作図
    inf = SxSys.getInf()
    layer0 = inf.active_layer   # 現在の書き込みレイヤーを取得

    opt = SxOptSelect()
    opt.ent_mode = SxSys.ENTMODE_SEG        # 検索モード定数:0: セグメント検索 
                                            # 要素(寸法、直線等)の検索モードを、[要]にする 
    opt.select_ent = True
    #opt.select_pos = True
    opt.select_seg = True
    opt.select_go = True
    
    (th_h, tau_h, rpm, layer, cam, offset) = get_cam()  # 諸条件とcam Object 取得
    t_h = abs(th_h)/(360*rpm/60)     # 実ストローク時間
    #t_h = th_h/(360*rpm/60)     # 実ストローク時間

    pm= th_h/abs(th_h)
    print('\nPCDをピックしてください')
    geom = SxSys.select(opt).ent.getGeom()     # マウスピックでセグメントのGeometoryを取得
    whl_c = np.array([geom.cp.x, geom.cp.y])    # Wheel Center
    whl_r = geom.radius

    print('\n揺動中心をピックしてください')
    tmp = SxSys.getPos()
    yd_c0 = np.array([tmp.x, tmp.y])  # 揺動中心

    yd_c_vec0 = yd_c0 - whl_c       #  Wheel Center -> Youdou Center Vector

    print('\nローラーをピックしてください')
    geom = SxSys.select(opt).ent.getGeom()     # マウスピックでセグメントのGeometoryを取得
    rol_r = geom.radius
    rol_c0 = np.array([geom.cp.x, geom.cp.y])   # ローラー中心座標 

    arm_vec0 = rol_c0 - yd_c0   # アームVector
    rol_c_vec0 = rol_c0 - whl_c     # Wheel Center -> Roler Center Vector 
    rol_vec0 = -rol_c_vec0/np.linalg.norm(rol_c_vec0) * rol_r   # ローラー中心->カム接点Vector

    rol_vec10 = rol_c_vec0/np.linalg.norm(rol_c_vec0) * offset   # ローラー中心->外接カム接点Vector

    #print(rol_vec0, rol_r)
    #nn = round(whl_r/12);
    #nn = 30
    #dth = th_h/nn
    dth = pm*0.01               # 2023.7.13
    nn = abs(round(th_h/dth))   # 2023.7.13

    while True:
        if nn >= 800:
            dth= dth*2
            nn = abs(round(th_h/dth))
        else:
            break

    #print('\n分割数: %d' % nn)
    sx_pnt_list = []        # Lorer中心座標
    sx_cam_list = []        # cam面の座標

    sx_cam_list1 = []        # 外接cam面の座標
    
    p_ang_list = []         # 圧力角 [deg]
    accel_list = []         # 加速度 [rad/sec^2]
    
    rat = offset/rol_r      # offset比率

    for i in range(nn+1):     # 0 - 10 まで
        t = i/nn    # 無次元時間
        (s, v, a) = cam.sva(t)          
        tau_rad = s*tau_h*np.pi/180     # 揺動角

        accel = a*tau_h/(t_h**2) * np.pi/180        # 実加速度[rad/sec^2]
        accel_list.append(accel)

        th_rad = i * dth * np.pi/180                # 公転角度
        yd_c_vec = myv.rotate2(yd_c_vec0, th_rad)   # 揺動Vector
        arm_vec = myv.rotate2(arm_vec0, th_rad+tau_rad) # アームVector
        rol_c = whl_c + yd_c_vec + arm_vec          # ローラー中心座標
        tmp_pos = SxPos(rol_c[0], rol_c[1])
        sx_pnt_list.append(tmp_pos)             # ローラー中心座標 SxPos配列
        
        if i == 0:
            rol_vec = rol_vec0              # ローラー半径Vector(接点)
            rol_vec1 = rol_vec10            # 外接ローラー半径Vector(接点)

            ctct_pnt = rol_c0 + rol_vec0    # 接点座標
            ctct_pnt1 = rol_c0 + rol_vec10    # 外接接点座標
            pre_rol_c = rol_c0              # 事前のローラー中心座標

        elif 0 < i < nn:
            tmp = rol_c - pre_rol_c         
            rol_k_vec = tmp/np.linalg.norm(tmp)     # ローラー中心軌跡Vector
            rol_vec90 = rol_r * myv.rotate2(rol_k_vec, np.pi/2)  # 90°回転
            tmp_p = rol_c + rol_vec90
            tmp_m = rol_c - rol_vec90
            #tmp_p = np.linalg.norm(rol_c + rol_vec90)
            #tmp_m = np.linalg.norm(rol_c - rol_vec90)
            #print(tmp_p, tmp_m, rol_vec90)
            if np.linalg.norm(tmp_p - whl_c) < np.linalg.norm(tmp_m - whl_c):
                rol_vec = rol_vec90     # Wheel Centerに近いほうを採用
                rol_vec1 = -rol_vec90*rat     # 外接カム
            else:
                rol_vec = -rol_vec90
                rol_vec1 = rol_vec90*rat

            ctct_pnt = rol_c + rol_vec  # ローラー、カム接点座標
            ctct_pnt1 = rol_c + rol_vec1  # ローラー、外接カム接点座標
            #sx_pos1 = SxPos(rol_c[0], rol_c[1])
            #sx_pos2 = SxPos(rol_c[0]+rol_vec[0], rol_c[1]+rol_vec[1])
            pre_rol_c = rol_c           # 事前のローラー中心座標
            #SxEntSeg.createLine2D(sx_pos1, sx_pos2)
        else:   # i == nn
            tmp = whl_c-rol_c 
            rol_vec = tmp/np.linalg.norm(tmp) * rol_r   # ローラー中心軌跡Vector 
            rol_vec1 = -tmp/np.linalg.norm(tmp) * offset   # ローラー中心軌跡Vector 
            ctct_pnt = rol_c + rol_vec  # ローラー、カム接点座標
            ctct_pnt1 = rol_c + rol_vec1  # ローラー、外接カム接点座標
        
        # debug用作図
        #imd.p2p_line(rol_c, rol_vec, '赤')

        th_in = np.arccos(arm_vec.dot(rol_vec) / np.linalg.norm(arm_vec) / np.linalg.norm(rol_vec)) #内角[rad]
        p_ang_list.append(abs(90 - th_in*180/np.pi))
        #print(th_in*180/np.pi)

        tmp_pos = SxPos(ctct_pnt[0], ctct_pnt[1])
        tmp_pos1 = SxPos(ctct_pnt1[0], ctct_pnt1[1])    # 外接カム
        
        sx_cam_list.append(tmp_pos)
        sx_cam_list1.append(tmp_pos1)   # 外接カム

    SxSys.setLayer(layer)   # 書き込みレイヤー設定
    #print(sx_pnt_list)  ####
    seg_lc = SxEntSeg.createSpline2D(sx_pnt_list,SxEntSeg.SPLINE_KIND_NORMAL,False,False,0.0,0.0)
    seg= SxEntRPart.create(seg_lc, 'lc', 0)

    seg_cam = SxEntSeg.createSpline2D(sx_cam_list,SxEntSeg.SPLINE_KIND_NORMAL,False,False,0.0,0.0)
    seg= SxEntRPart.create(seg_cam, 'cam', 0)

    # 外接カム
    if offset != 0:
        seg_cam1 = SxEntSeg.createSpline2D(sx_cam_list1,SxEntSeg.SPLINE_KIND_NORMAL,False,False,0.0,0.0)
        seg= SxEntRPart.create(seg_cam1, 'cam1', 0)
            
    SxSys.setLayer(layer0)  # レイヤーを元に戻す

    SxEnt.setColor(seg_lc, SxSys.COLOR_SKY_BLUE)   # 空色
    SxEnt.setLineStyle(seg_lc, SxSys.STYLE_ONE_DOT_CHAIN)  #一点鎖線
    SxEnt.setLineWidth(seg_lc, SxSys.WIDTH_THIN)  # 細線

    SxEnt.setColor(seg_cam, SxSys.COLOR_ORANGE)     # 橙
    SxEnt.setLineStyle(seg_cam, SxSys.STYLE_SOLID)   # 実線
    SxEnt.setLineWidth(seg_cam, SxSys.WIDTH_MIDDLE)   # 中線
    if offset != 0:
        SxEnt.setColor(seg_cam1, SxSys.COLOR_ORANGE)     # 橙
        SxEnt.setLineStyle(seg_cam1, SxSys.STYLE_SOLID)   # 実線
        SxEnt.setLineWidth(seg_cam1, SxSys.WIDTH_MIDDLE)   # 中線
    
    print('\n   分割数：%d で作図を完了しました\007\n' % nn)
    
    # 最大圧力角
    ss_max_ang = '%3.2f' % (max(p_ang_list))    # 最大圧力角
    #e_max_ang.delete(0,tk.END)  # エントリー中身を削除
    #e_max_ang.insert(0,ss)     # エントリーの中身を設定

    # 最大加速度
    ss_max_accel = '%3.2f' % (max(accel_list))    # 最大加速度
    #e_max_accel.delete(0,tk.END)  # エントリー中身を削除
    #e_max_accel.insert(0,ss)     # エントリーの中身を設定
    
    print_inf(ss_max_ang, ss_max_accel)



#--------------------------------------------------
#   画面設定
#--------------------------------------------------
root = tk.Tk()
root.attributes("-topmost", True)    # 常に前面に表示
root.configure(bg='RoyalBlue4')
root.title('icd_grp.py Ver.1.0')   #タイトル

#--------------------------------------------------
# ボタン等の部品配置
#--------------------------------------------------
btn_bcol = 'grey21'     #ボタン背景色
btn_tcol = 'white'      #ボタン文字色
label_bcol = 'khaki3'   #ラベル背景色
label_tcol = 'black'   #ラベル文字色
e_bcol = 'khaki4'
x00 = 10; y00 = 5
dx = 55;  dy = 30
wdth = 10; wdth1 = 6
nx = 1.9; ny = 8
ss_size = '%dx%d+%d+%d' % (x00+dx*(nx+2)+10, y00+dy*ny, 1100, 0)
root.geometry(ss_size)


#-------------------------------------------------------
ss = 'ヘルプ';  nx = 2.9; ny = 0    # Buttn
#-------------------------------------------------------
btn = tk.Button(root,text = ss,command = push_help, bg=btn_bcol,fg=btn_tcol) #ボタンの作成
btn.place(x=x00+nx*dx,y=y00+ny*dy)

#-------------------------------------------------------
ss = 'Graph';  nx = 2.9; ny = 1    # Buttn
#-------------------------------------------------------
btn = tk.Button(root,text = ss,command = push_graph, bg=btn_bcol,fg=btn_tcol) #ボタンの作成
btn.place(x=x00+nx*dx,y=y00+ny*dy)


#---------------------------------------------------
ss = '全割付角[deg]' ; nx = 0; ny = 0  # Label & Entory
#---------------------------------------------------
label = tk.Label(root,text=ss ,bg=label_bcol)  # Labelの作成
label.place(x=x00+nx*dx,y=y00+ny*dy)
e_th_h= tk.Entry(root, width=wdth1, bg=e_bcol)  # エントリー部品の生成
e_th_h.place(x=x00+(nx+2)*dx,y=y00+ny*dy)
e_th_h.insert(0,'15')     # エントリーの中身を設定


#---------------------------------------------------
ss = '全揺動角[deg] ' ; nx = 0; ny = 1  # Label & Entory
#---------------------------------------------------
label = tk.Label(root,text=ss ,bg=label_bcol)  # Labelの作成
label.place(x=x00+nx*dx,y=y00+ny*dy)
e_tau_h= tk.Entry(root, width=wdth1, bg=e_bcol)  # エントリー部品の生成
e_tau_h.place(x=x00+(nx+2)*dx,y=y00+ny*dy)
e_tau_h.insert(0,'15')     # エントリーの中身を設定

#---------------------------------------------------
ss = 'wheel回転数[rpm]' ; nx = 0; ny = 2  # Label & Entory
#---------------------------------------------------
label = tk.Label(root,text=ss ,bg=label_bcol)  # Labelの作成
label.place(x=x00+nx*dx,y=y00+ny*dy)
e_rpm= tk.Entry(root, width=wdth1, bg=e_bcol)  # エントリー部品の生成
e_rpm.place(x=x00+(nx+2)*dx,y=y00+ny*dy)
e_rpm.insert(0,'25')     # エントリーの中身を設定


#---------------------------------------------------
ss = '外接cam offset[mm]' ; nx = 0; ny = 3  # Label & Entory
#---------------------------------------------------
label = tk.Label(root,text=ss ,bg=label_bcol)  # Labelの作成
label.place(x=x00+nx*dx,y=y00+ny*dy)
e_offset= tk.Entry(root, width=wdth1, bg=e_bcol)  # エントリー部品の生成
e_offset.place(x=x00+(nx+2.2)*dx,y=y00+ny*dy)
e_offset.insert(0,'0')     # エントリーの中身を設定


#---------------------------------------------------
ss = 'カ ム 曲 線' ; nx = 0; ny = 4  # Label & Entory
#---------------------------------------------------
label = tk.Label(root,text=ss ,bg=label_bcol)  # Labelの作成
label.place(x=x00+nx*dx,y=y00+ny*dy)

v_cam = tk.StringVar()
cb = ttk.Combobox(root, width=wdth,textvariable = v_cam) #, style="BW.TLabel")
#cb.bind('<<ComboboxSelected>>', cb_selected_zai)
cb['values']=('変形正弦','変形台形','変形等速度','Cycroidal')
cb.set('変形正弦')
cb.place(x=x00+(nx+1.3*dx),y=y00+ny*dy)

#---------------------------------------------------
ss = '作図レイヤ' ; nx = 0; ny = 5 # Label & Entory
#---------------------------------------------------
label = tk.Label(root,text=ss ,bg=label_bcol)  # Labelの作成
label.place(x=x00+nx*dx,y=y00+ny*dy)
e_layer= tk.Entry(root, width=wdth1, bg=e_bcol)  # エントリー部品の生成
e_layer.place(x=x00+(nx+1.3)*dx,y=y00+ny*dy)
e_layer.insert(0,'10')     # エントリーの中身を設定

#-------------------------------------------------------
ss = 'Cam作図';  nx = 2.9; ny = 4    # Buttn
#-------------------------------------------------------
btn = tk.Button(root,text = ss,command = push_sakuzu, bg=btn_bcol,fg=btn_tcol) #ボタンの作成
btn.place(x=x00+nx*dx,y=y00+ny*dy-3)


#---------------------------------------------------
ss = '公転角 θ' ; nx = 0; ny = 6  # Label & Entory & button
#---------------------------------------------------
label = tk.Label(root,text=ss ,bg=label_bcol)  # Labelの作成
label.place(x=x00+nx*dx,y=y00+ny*dy)
e_th= tk.Entry(root, width=8,bg=e_bcol)  # エントリー部品の生成
e_th.place(x=x00+(nx+1.3)*dx,y=y00+ny*dy)
e_th.insert(0,'5')     # エントリーの中身を設定
ss = 'θ->τ 計算';  nx = 2.5    # Buttn
btn = tk.Button(root,text = ss, command = push_th2tau, bg=btn_bcol,fg=btn_tcol) #ボタンの作成
btn.place(x=x00+nx*dx,y=y00+ny*dy-3)

#---------------------------------------------------
ss = '揺動角 τ' ; nx = 0; ny = 7  # Label & Entory & button
#---------------------------------------------------
label = tk.Label(root,text=ss ,bg=label_bcol)  # Labelの作成
label.place(x=x00+nx*dx,y=y00+ny*dy)
e_tau= tk.Entry(root, width=8,bg=e_bcol)  # エントリー部品の生成
e_tau.place(x=x00+(nx+1.3)*dx,y=y00+ny*dy)
e_th.insert(0,'')     # エントリーの中身を設定
ss = 'τ->θ 計算';  nx = 2.5    # Buttn
btn = tk.Button(root,text = ss, command = push_tau2th, bg=btn_bcol,fg=btn_tcol) #ボタンの作成
btn.place(x=x00+nx*dx,y=y00+ny*dy-3)

"""
#---------------------------------------------------
ss = '最大圧力角[deg]' ; nx = 0; ny = 7  # Label & Entory
#---------------------------------------------------
label = tk.Label(root,text=ss ,bg='orange')  # Labelの作成
label.place(x=x00+nx*dx,y=y00+ny*dy)
e_max_ang = tk.Entry(root, width=wdth,bg=e_bcol)  # エントリー部品の生成
e_max_ang.place(x=x00+(nx+2.5)*dx,y=y00+ny*dy)
#e_layer.insert(0,'')     # エントリーの中身を設定


#---------------------------------------------------
ss = '最大加速度[rad/sec^2]' ; nx = 0; ny = 8  # Label & Entory
#---------------------------------------------------
label = tk.Label(root,text=ss ,bg='orange')  # Labelの作成
label.place(x=x00+nx*dx,y=y00+ny*dy)
e_max_accel= tk.Entry(root, width=wdth,bg=e_bcol)  # エントリー部品の生成
e_max_accel.place(x=x00+(nx+2.5)*dx,y=y00+ny*dy)
"""



root.mainloop()
